#!/usr/bin/env python3
import sys
import os
import os.path
from typing import List

python_ver = '{}.{}'.format(*sys.version_info[:2])
sys.path.insert(0, f'/usr/share/kcare-eportal/vendor/lib/python{python_ver}/site-packages')
sys.path.insert(0, f'/usr/share/kcare-eportal/vendor/lib64/python{python_ver}/site-packages')

import glob
import shutil
from argparse import ArgumentParser
from alembic.config import main as alembic

import config
from eportal import utils


def get_file_uid(path: str) -> int:
    return os.stat(path).st_uid


def get_sqlite_file_pathes() -> List[str]:
    return glob.glob(os.path.join(config.BASE_DIR, '*.sqlite*'))


def main() -> None:
    parser = ArgumentParser(description='Migrate database')
    parser.add_argument('--upgrade', action='store_true', help='Do DB upgrade only')
    parser.add_argument('--skip-chown', action='store_true', help='Skip chown operation for DB files')
    args = parser.parse_args()

    if not args.skip_chown:
        # fix db permissions
        uid, gid = utils.get_base_owner()
        for f in get_sqlite_file_pathes():
            shutil.chown(f, uid, gid)

    utils.become_owner()

    proc_euid = os.geteuid()
    for path in [config.BASE_DIR] + get_sqlite_file_pathes():
        file_uid = get_file_uid(path)
        if file_uid != proc_euid:
            print(f'ERROR: file owner mismatch for {path}. Expected {proc_euid}, got {file_uid}')
            sys.exit(1)

    # do these import after become_owner to be sure all databases would be
    # created with proper owner
    from eportal import oplog

    print('Creating oplog table ... ', end='')
    oplog.create_table()
    print('done.')

    cfg_path = os.path.join(os.path.dirname(__file__), 'alembic.ini')
    cfg_args = ['--config', cfg_path]

    alembic(cfg_args + ['upgrade', 'head'])
    alembic(cfg_args + ['-n', 'patches-data', 'upgrade', 'head'])
    alembic(cfg_args + ['-n', 'spooler', 'upgrade', 'head'])


if __name__ == '__main__':
    main()
