"""migrate data

Revision ID: db8961daf086
Revises: f97011184c10
Create Date: 2024-02-23 08:35:55.054282

"""

# revision identifiers, used by Alembic.
revision = 'db8961daf086'
down_revision = 'f97011184c10'

import sys
from alembic import op
from sqlbind import FIELDS


def copy_data(stream, session):
    from eportal.dbutils import transaction

    dbpath = op.get_bind().engine.url.database

    with transaction(session):
        q = session.query_params()

        tables = q.execute('PRAGMA main.table_list').column(1)
        if not set(tables).issuperset({'releases', 'patches', 'fixups'}):
            print(f'No data for stream {stream}', file=sys.stderr)
            return

        q.execute(f'ATTACH DATABASE {q/dbpath} AS combined')

        fields = ['release', 'kernel_id', 'patch_level', 'version']
        q.execute(f'REPLACE INTO combined.patches ({FIELDS(*fields)}) SELECT {FIELDS(*fields)} from patches')

        fields = ['release', 'kernel_id', 'fixup_for', 'fixup_from']
        q.execute(f'REPLACE INTO combined.fixups ({FIELDS(*fields)}) SELECT {FIELDS(*fields)} from fixups')

        fields = ['release', 'feed', 'deployed', 'enabled']
        q.execute(
            f'''\
            REPLACE INTO combined.releases (stream, {FIELDS(*fields)})
            SELECT {q/stream} as stream, {FIELDS(*fields)} from releases
            '''
        )


def upgrade():
    from eportal.patches import legacy_dbs

    copy_data('db', legacy_dbs.db.session)
    copy_data('qemu', legacy_dbs.qemu.session)
    copy_data('user', legacy_dbs.user.session)
    copy_data('kernel', legacy_dbs.kernel.session)


def downgrade():
    pass
